import numpy as np
import matplotlib.pyplot as plt
import os
# Make sure that caffe is on the python path:
import sys 
CAFFE_ROOT = '../../Caffe-Repo' # CHANGE THIS LINE TO YOUR Caffe PATH
sys.path.insert(0, CAFFE_ROOT + 'python')

import caffe

# Set the right path to your model definition file, pretrained model weights,
# and the image you would like to classify.
MODEL_FILE = 'deploy.prototxt'
PRETRAINED = 'models/test02__iter_12000.caffemodel' #Weights to be used w/MODEL_FILE

# Use GPU or CPU
caffe.set_mode_cpu()
#caffe.set_mode_gpu()

# Load network
# Note arguments to preprocess input
#  mean subtraction switched on by giving a mean array
#  input channel swapping takes care of mapping RGB into the reference ImageNet model's BGR order
#  raw scaling multiplies the feature scale from the input [0,1] to the ImageNet model's [0,255]

net = caffe.Classifier(MODEL_FILE, PRETRAINED,
                      mean=np.load('images/train_mean.npy').mean(1).mean(1),
                      #channel_swap=(0,1,2),
                      raw_scale=255,
                      image_dims=(24, 24))
# Alternatively, the above could have been written as:
#net = caffe.Classifier(MODEL_FILE, PRETRAINED)
#net.transformer.set_mean('data', np.load('train_mean.npy').mean(1).mean(1))
#net.transformer.set_raw_scale('data', 255)  # reference model operates on [0,255] range instead of [0,1]
#net.transformer.set_channel_swap('data', (0,1,2))  # the reference model has channels in BGR order instead of RGB
#net.transformer.set_image_dims('data', (24,24))
net.blobs["data"].reshape(1, 1, 24, 24)
IMAGES_FOLDER = 'validate_ball/'

images = os.listdir(IMAGES_FOLDER)
input_images = [ caffe.io.load_image(IMAGES_FOLDER + im, color=False) for im in images ]


# Classify image
prediction = net.predict(input_images)  # predict takes any number of images, and formats them for the Caffe net automatically
#print 'predicted classes:', prediction[0].argmax(), prediction[1].argmax()

classBall =  np.round(prediction[:,1],0)
classNoBall = np.round(prediction[:,0],0)

unique_ball, counts_ball = np.unique(classBall, return_counts=True)
unique_noball, counts_noball = np.unique(classNoBall, return_counts=True)
print("\n")
print("Balls detected in " + IMAGES_FOLDER)
print(dict(zip(unique_ball, counts_ball)))
print("\n")
print("NoBalls detected in " + IMAGES_FOLDER)
print(dict(zip(unique_noball, counts_noball)))


### The steps below are OPTIONAL ###
# View input_image1
#plt.imshow(input_image1)
#plt.show()
# View the probabilities of all the classes for input_image1 as a bar chart
#plt.plot(prediction[0])
#plt.show()
# Time the full pipeline classification for 1 image w/oversampling
#net.predict([input_image1])

# Time the only forward pass classification for 1 image by resizing/oversampling before forward pass
#   Resize the image to the standard (256, 256) and oversample net input sized crops.
#input_oversampled = caffe.io.oversample([caffe.io.resize_image(input_image1, net.image_dims)], net.crop_dims)
#   'data' is the input blob name in the model definition, so we preprocess for that input.
#caffe_input = np.asarray([net.transformer.preprocess('data', in_) for in_ in input_oversampled])
#  forward() takes keyword args for the input blobs with preprocessed input arrays.
#net.forward(data=caffe_input)